; serial_message_silabs_nov10.a51 	sends a message to pc using UART--and a value (P2, treated as ascii value)
; Nov 10: splices june 10 program with initializations done by Wizard Nov. 10
; Sept 2019 changed name of routine from GETMESSAGE to GETSPEECH, to match 
; language of LAoE text
	
; rev path 3/16
	$NOSYMBOLS			; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file

	STACKBOT EQU 80h					; put stack at start of scratch indirectly-addressable block (80h and up)

	TIMER96K EQU 96h		; timer reload value for 9600 baud---assuming SiLabs, 24MHz div-by-12	(p.214) 
											;  sys clock at full speed

	SAY EQU 00h				; command says next characters (till ;) are speech codes
	EOM EQU ';'   		; EOM for ascii mode
        LF  EQU 0Ah				; linefeed
  
	DISPLAY EQU P1
	KEYPAD EQU P2
	
	START_OFFSET EQU 01h	; initial offset into speech table (may have to tinker, to get it right)

	ORG 0h
	LJMP STARTUP

	ORG 080h
				
STARTUP: MOV SP, #STACKBOT-1

	acall USUAL_SETUP
	acall Init_Device



;* * * * * * * * * * * * *	transmit code


REINIT:	MOV R2, #START_OFFSET		; set up offset into table of characters

	
MORE:	MOV A, R2			; pick up offset into speech table
	ACALL GETSPEECH			; Now start getting speech, a character at a time 
	CJNE A, #EOM, STILL_OK	; check for end of message (EOM is semicolon, in this program)


; now send data, after message: this is a single character, taken from controller keypad 

  mov A, P2						; get keypad value, just to have something to send
	acall SENDIT
; now a newline		
	mov A, #0Dh         ; carriage return
	acall SENDIT
	mov A, #LF					; linefeed
	acall SENDIT

;	sjmp 	QUIT 					; say it just once
	SJMP REINIT				; ...or keep saying it, over and over
	
STILL_OK:	ACALL SENDIT		; if not end, send to talker
	MOV A, R2				; pick up current offset value
	INC A 					; advance to next character
	JZ OVERFLOW				; check for overflow of 8-bit offset
	MOV R2, A				; ...if no overflow, update offset value
	SJMP MORE
	

OVERFLOW: CLR P2.1		; in case of overflow, light LED...
QUIT: SJMP QUIT			; ...and hang up here, till reset


;* * * * sends a byte using UART0, when Talker isn't busy

SENDIT: 
	MOV	SBUF0, A					; Put character in buffer (send it)--UART 1
LINGER:	JNB TI0, LINGER		; wait here till told it's been sent
	CLR TI0
   RET


; * * * THE SPEECH ITSELF
	ORG 0480h
GETSPEECH: MOVC A, @A+PC
	RET 

	db 'this is the 410 talking to a PC through its serial port. This is P2 value, as ascii:;'

;--- INITIALIZATIONS -----

USUAL_SETUP: anl   PCA0MD, #NOT(040h)        ; Disable the WDT.
                                             ; Clear Watchdog Enable bit
				; Configure the Oscillator
			mov OSCICN, #087h										; sysclk = 24.5 MHz

         ; Enable the Port I/O Crossbar
            mov   XBR1, #40h           ; Enable Crossbar
						ret
;------------------------------------
;-  Generated Initialization File  --
;------------------------------------

; Peripheral specific initialization functions,
; Called from the Init_Device label
Timer_Init:
    mov  TCON,      #040h
    mov  TMOD,      #020h
;    mov  CKCON,     #008h   ; use system clock (try removing this , which was not in earlier pgm)
    mov  TH1,       #0F6h
    mov TH1, #096h   ; this value taken from non-Wizard version of this program
    ret

UART_Init:
    mov  SCON0,     #010h
    ret

Port_IO_Init:
    ; P0.0  -  Unassigned,  Open-Drain, Digital
    ; P0.1  -  Unassigned,  Open-Drain, Digital
    ; P0.2  -  Unassigned,  Open-Drain, Digital
    ; P0.3  -  Unassigned,  Open-Drain, Digital
    ; P0.4  -  TX0 (UART0), Push-Pull,  Digital
    ; P0.5  -  RX0 (UART0), Open-Drain, Digital
    ; P0.6  -  Unassigned,  Open-Drain, Digital
    ; P0.7  -  Unassigned,  Open-Drain, Digital

    ; P1.0  -  Unassigned,  Open-Drain, Digital
    ; P1.1  -  Unassigned,  Open-Drain, Digital
    ; P1.2  -  Unassigned,  Open-Drain, Digital
    ; P1.3  -  Unassigned,  Open-Drain, Digital
    ; P1.4  -  Unassigned,  Open-Drain, Digital
    ; P1.5  -  Unassigned,  Open-Drain, Digital
    ; P1.6  -  Unassigned,  Open-Drain, Digital
    ; P1.7  -  Unassigned,  Open-Drain, Digital

    ; P2.0  -  Unassigned,  Open-Drain, Digital
    ; P2.1  -  Unassigned,  Open-Drain, Digital
    ; P2.2  -  Unassigned,  Open-Drain, Digital
    ; P2.3  -  Unassigned,  Open-Drain, Digital
    ; P2.4  -  Unassigned,  Open-Drain, Digital
    ; P2.5  -  Unassigned,  Open-Drain, Digital
    ; P2.6  -  Unassigned,  Open-Drain, Digital
    ; P2.7  -  Unassigned,  Open-Drain, Digital

    mov  P0MDOUT,   #010h
    mov  XBR0,      #001h
    mov  XBR1,      #040h
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
    lcall Timer_Init
    lcall UART_Init
    lcall Port_IO_Init
    ret

end
	
